@@ -7,11 +7,6 @@ import os
 import numpy as np
 from collections import defaultdict
 from matplotlib import pyplot
-
-import re
-
-from argparse import Namespace
-
 from faiss.contrib.factory_tools import get_code_size as unitsize
 
 
@@ -141,7 +136,7 @@ def collect_results_for(db='deep1M', prefix="autotune."):
             print("FAIL %s" % e)
             res = np.zeros((2, 0))
         except Exception as e:
-            print("PARSE ERROR " + e)
+            print("PARSE ERROR " + str(e))
             res = np.zeros((2, 0))
         else:
             print(len(res), "results")
@@ -158,6 +153,7 @@ def collect_results_for(db='deep1M', prefix="autotune."):
 
     return allres, allstats
 
+
 def extract_pareto_optimal(allres, keys, recall_idx=0, times_idx=3):
     bigtab = []
     for i, k in enumerate(keys):
@@ -188,6 +184,7 @@ def extract_pareto_optimal(allres, keys, recall_idx=0, times_idx=3):
 
     return selected_keys, ops
 
+
 def plot_subset(
     allres, allstats, selected_methods, recall_idx, times_idx=3,
     report=["overhead", "build time"]):
@@ -262,14 +259,14 @@ def plot_tradeoffs(db, allres, allstats, code_size, recall_rank):
     # times come after the perf measure
     times_idx = len(stat0["ranks"])
 
-    if type(code_size) == int:
+    if isinstance(code_size, int):
         if code_size == 0:
             code_size = [0, 1e50]
             code_size_name = "any code size"
         else:
             code_size_name = "code_size=%d" % code_size
             code_size = [code_size, code_size]
-    elif type(code_size) == tuple:
+    elif isinstance(code_size, tuple):
         code_size_name = "code_size in [%d, %d]" % code_size
     else:
         assert False
