@@ -221,9 +221,9 @@ static void verifyIndex2LevelDecoder(
 
     // generate a random order of points
     std::uniform_int_distribution<uint64_t> un(0, n - 1);
-    std::vector<uint64_t> pointIncidesToDecode(nIterations * n, 0);
+    std::vector<uint64_t> pointIndicesToDecode(nIterations * n, 0);
     for (uint64_t i = 0; i < nIterations * n; i++) {
-        pointIncidesToDecode[i] = un(rng);
+        pointIndicesToDecode[i] = un(rng);
     }
 
     {
@@ -234,7 +234,7 @@ static void verifyIndex2LevelDecoder(
         StopWatch swFaiss;
         for (uint64_t iter = 0; iter < nIterations; iter++) {
             for (uint64_t i = 0; i < n; i++) {
-                const auto pointIdx = pointIncidesToDecode[i + iter * n];
+                const auto pointIdx = pointIndicesToDecode[i + iter * n];
                 index->sa_decode(
                         1,
                         encodedData.data() + pointIdx * codeSize,
@@ -247,7 +247,7 @@ static void verifyIndex2LevelDecoder(
         StopWatch swKernel;
         for (uint64_t iter = 0; iter < nIterations; iter++) {
             for (uint64_t i = 0; i < n; i++) {
-                const auto pointIdx = pointIncidesToDecode[i + iter * n];
+                const auto pointIdx = pointIndicesToDecode[i + iter * n];
                 T::store(
                         pqCoarseCentroidsQ,
                         pqFineCentroidsQ,
@@ -289,7 +289,7 @@ static void verifyIndex2LevelDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter++) {
                 const auto pointIdx =
-                        pointIncidesToDecode[i * nIterations + iter];
+                        pointIndicesToDecode[i * nIterations + iter];
                 const auto weight = weights[i * nIterations + iter];
                 index->sa_decode(
                         1,
@@ -307,7 +307,7 @@ static void verifyIndex2LevelDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter++) {
                 const auto pointIdx =
-                        pointIncidesToDecode[i * nIterations + iter];
+                        pointIndicesToDecode[i * nIterations + iter];
                 const auto weight = weights[i * nIterations + iter];
                 T::accum(
                         pqCoarseCentroidsQ,
@@ -331,10 +331,10 @@ static void verifyIndex2LevelDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter += 2) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 T::accum(
                         pqCoarseCentroidsQ,
@@ -360,10 +360,10 @@ static void verifyIndex2LevelDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter += 2) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 T::accum(
                         pqCoarseCentroidsQ,
@@ -391,13 +391,13 @@ static void verifyIndex2LevelDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter += 3) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 const auto pointIdx2 =
-                        pointIncidesToDecode[i * nIterations + iter + 2];
+                        pointIndicesToDecode[i * nIterations + iter + 2];
                 const auto weight2 = weights[i * nIterations + iter + 2];
                 T::accum(
                         pqCoarseCentroidsQ,
@@ -425,13 +425,13 @@ static void verifyIndex2LevelDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter += 3) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 const auto pointIdx2 =
-                        pointIncidesToDecode[i * nIterations + iter + 2];
+                        pointIndicesToDecode[i * nIterations + iter + 2];
                 const auto weight2 = weights[i * nIterations + iter + 2];
                 T::accum(
                         pqCoarseCentroidsQ,
@@ -532,9 +532,9 @@ static void verifyMinMaxIndex2LevelDecoder(
 
     // generate a random order of points
     std::uniform_int_distribution<uint64_t> un(0, n - 1);
-    std::vector<uint64_t> pointIncidesToDecode(nIterations * n, 0);
+    std::vector<uint64_t> pointIndicesToDecode(nIterations * n, 0);
     for (uint64_t i = 0; i < nIterations * n; i++) {
-        pointIncidesToDecode[i] = un(rng);
+        pointIndicesToDecode[i] = un(rng);
     }
 
     {
@@ -545,7 +545,7 @@ static void verifyMinMaxIndex2LevelDecoder(
         StopWatch swFaiss;
         for (uint64_t iter = 0; iter < nIterations; iter++) {
             for (uint64_t i = 0; i < n; i++) {
-                const auto pointIdx = pointIncidesToDecode[i + iter * n];
+                const auto pointIdx = pointIndicesToDecode[i + iter * n];
                 index->sa_decode(
                         1,
                         encodedData.data() + pointIdx * codeSize,
@@ -558,7 +558,7 @@ static void verifyMinMaxIndex2LevelDecoder(
         StopWatch swKernel;
         for (uint64_t iter = 0; iter < nIterations; iter++) {
             for (uint64_t i = 0; i < n; i++) {
-                const auto pointIdx = pointIncidesToDecode[i + iter * n];
+                const auto pointIdx = pointIndicesToDecode[i + iter * n];
                 T::store(
                         pqCoarseCentroidsQ,
                         pqFineCentroidsQ,
@@ -601,7 +601,7 @@ static void verifyMinMaxIndex2LevelDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter++) {
                 const auto pointIdx =
-                        pointIncidesToDecode[i * nIterations + iter];
+                        pointIndicesToDecode[i * nIterations + iter];
                 const auto weight = weights[i * nIterations + iter];
                 index->sa_decode(
                         1,
@@ -620,7 +620,7 @@ static void verifyMinMaxIndex2LevelDecoder(
             float outputAccumMin = 0;
             for (uint64_t iter = 0; iter < nIterations; iter++) {
                 const auto pointIdx =
-                        pointIncidesToDecode[i * nIterations + iter];
+                        pointIndicesToDecode[i * nIterations + iter];
                 const auto weight = weights[i * nIterations + iter];
                 T::accum(
                         pqCoarseCentroidsQ,
@@ -649,10 +649,10 @@ static void verifyMinMaxIndex2LevelDecoder(
             float outputAccumMin = 0;
             for (uint64_t iter = 0; iter < nIterations; iter += 2) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 T::accum(
                         pqCoarseCentroidsQ,
@@ -683,10 +683,10 @@ static void verifyMinMaxIndex2LevelDecoder(
             float outputAccumMin = 0;
             for (uint64_t iter = 0; iter < nIterations; iter += 2) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 T::accum(
                         pqCoarseCentroidsQ,
@@ -719,13 +719,13 @@ static void verifyMinMaxIndex2LevelDecoder(
             float outputAccumMin = 0;
             for (uint64_t iter = 0; iter < nIterations; iter += 3) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 const auto pointIdx2 =
-                        pointIncidesToDecode[i * nIterations + iter + 2];
+                        pointIndicesToDecode[i * nIterations + iter + 2];
                 const auto weight2 = weights[i * nIterations + iter + 2];
                 T::accum(
                         pqCoarseCentroidsQ,
@@ -758,13 +758,13 @@ static void verifyMinMaxIndex2LevelDecoder(
             float outputAccumMin = 0;
             for (uint64_t iter = 0; iter < nIterations; iter += 3) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 const auto pointIdx2 =
-                        pointIncidesToDecode[i * nIterations + iter + 2];
+                        pointIndicesToDecode[i * nIterations + iter + 2];
                 const auto weight2 = weights[i * nIterations + iter + 2];
                 T::accum(
                         pqCoarseCentroidsQ,
@@ -859,9 +859,9 @@ static void verifyIndexPQDecoder(
 
     // generate a random order of points
     std::uniform_int_distribution<uint64_t> un(0, n - 1);
-    std::vector<uint64_t> pointIncidesToDecode(nIterations * n, 0);
+    std::vector<uint64_t> pointIndicesToDecode(nIterations * n, 0);
     for (uint64_t i = 0; i < nIterations * n; i++) {
-        pointIncidesToDecode[i] = un(rng);
+        pointIndicesToDecode[i] = un(rng);
     }
 
     {
@@ -872,7 +872,7 @@ static void verifyIndexPQDecoder(
         StopWatch swFaiss;
         for (uint64_t iter = 0; iter < nIterations; iter++) {
             for (uint64_t i = 0; i < n; i++) {
-                const auto pointIdx = pointIncidesToDecode[i + iter * n];
+                const auto pointIdx = pointIndicesToDecode[i + iter * n];
                 index->sa_decode(
                         1,
                         encodedData.data() + pointIdx * codeSize,
@@ -885,7 +885,7 @@ static void verifyIndexPQDecoder(
         StopWatch swKernel;
         for (uint64_t iter = 0; iter < nIterations; iter++) {
             for (uint64_t i = 0; i < n; i++) {
-                const auto pointIdx = pointIncidesToDecode[i + iter * n];
+                const auto pointIdx = pointIndicesToDecode[i + iter * n];
                 T::store(
                         pqFineCentroidsQ,
                         encodedData.data() + pointIdx * codeSize,
@@ -927,7 +927,7 @@ static void verifyIndexPQDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter++) {
                 const auto pointIdx =
-                        pointIncidesToDecode[i * nIterations + iter];
+                        pointIndicesToDecode[i * nIterations + iter];
                 const auto weight = weights[i * nIterations + iter];
                 index->sa_decode(
                         1,
@@ -945,7 +945,7 @@ static void verifyIndexPQDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter++) {
                 const auto pointIdx =
-                        pointIncidesToDecode[i * nIterations + iter];
+                        pointIndicesToDecode[i * nIterations + iter];
                 const auto weight = weights[i * nIterations + iter];
                 T::accum(
                         pqFineCentroidsQ,
@@ -968,10 +968,10 @@ static void verifyIndexPQDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter += 2) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 T::accum(
                         pqFineCentroidsQ,
@@ -996,10 +996,10 @@ static void verifyIndexPQDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter += 2) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 T::accum(
                         pqFineCentroidsQ,
@@ -1025,13 +1025,13 @@ static void verifyIndexPQDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter += 3) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 const auto pointIdx2 =
-                        pointIncidesToDecode[i * nIterations + iter + 2];
+                        pointIndicesToDecode[i * nIterations + iter + 2];
                 const auto weight2 = weights[i * nIterations + iter + 2];
                 T::accum(
                         pqFineCentroidsQ,
@@ -1058,13 +1058,13 @@ static void verifyIndexPQDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter += 3) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 const auto pointIdx2 =
-                        pointIncidesToDecode[i * nIterations + iter + 2];
+                        pointIndicesToDecode[i * nIterations + iter + 2];
                 const auto weight2 = weights[i * nIterations + iter + 2];
                 T::accum(
                         pqFineCentroidsQ,
@@ -1157,9 +1157,9 @@ static void verifyMinMaxIndexPQDecoder(
 
     // generate a random order of points
     std::uniform_int_distribution<uint64_t> un(0, n - 1);
-    std::vector<uint64_t> pointIncidesToDecode(nIterations * n, 0);
+    std::vector<uint64_t> pointIndicesToDecode(nIterations * n, 0);
     for (uint64_t i = 0; i < nIterations * n; i++) {
-        pointIncidesToDecode[i] = un(rng);
+        pointIndicesToDecode[i] = un(rng);
     }
 
     {
@@ -1170,7 +1170,7 @@ static void verifyMinMaxIndexPQDecoder(
         StopWatch swFaiss;
         for (uint64_t iter = 0; iter < nIterations; iter++) {
             for (uint64_t i = 0; i < n; i++) {
-                const auto pointIdx = pointIncidesToDecode[i + iter * n];
+                const auto pointIdx = pointIndicesToDecode[i + iter * n];
                 index->sa_decode(
                         1,
                         encodedData.data() + pointIdx * codeSize,
@@ -1183,7 +1183,7 @@ static void verifyMinMaxIndexPQDecoder(
         StopWatch swKernel;
         for (uint64_t iter = 0; iter < nIterations; iter++) {
             for (uint64_t i = 0; i < n; i++) {
-                const auto pointIdx = pointIncidesToDecode[i + iter * n];
+                const auto pointIdx = pointIndicesToDecode[i + iter * n];
                 T::store(
                         pqFineCentroidsQ,
                         encodedData.data() + pointIdx * codeSize,
@@ -1225,7 +1225,7 @@ static void verifyMinMaxIndexPQDecoder(
         for (uint64_t i = 0; i < n; i++) {
             for (uint64_t iter = 0; iter < nIterations; iter++) {
                 const auto pointIdx =
-                        pointIncidesToDecode[i * nIterations + iter];
+                        pointIndicesToDecode[i * nIterations + iter];
                 const auto weight = weights[i * nIterations + iter];
                 index->sa_decode(
                         1,
@@ -1244,7 +1244,7 @@ static void verifyMinMaxIndexPQDecoder(
             float outputAccumMin = 0;
             for (uint64_t iter = 0; iter < nIterations; iter++) {
                 const auto pointIdx =
-                        pointIncidesToDecode[i * nIterations + iter];
+                        pointIndicesToDecode[i * nIterations + iter];
                 const auto weight = weights[i * nIterations + iter];
                 T::accum(
                         pqFineCentroidsQ,
@@ -1272,10 +1272,10 @@ static void verifyMinMaxIndexPQDecoder(
             float outputAccumMin = 0;
             for (uint64_t iter = 0; iter < nIterations; iter += 2) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 T::accum(
                         pqFineCentroidsQ,
@@ -1305,10 +1305,10 @@ static void verifyMinMaxIndexPQDecoder(
             float outputAccumMin = 0;
             for (uint64_t iter = 0; iter < nIterations; iter += 2) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 T::accum(
                         pqFineCentroidsQ,
@@ -1339,13 +1339,13 @@ static void verifyMinMaxIndexPQDecoder(
             float outputAccumMin = 0;
             for (uint64_t iter = 0; iter < nIterations; iter += 3) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 const auto pointIdx2 =
-                        pointIncidesToDecode[i * nIterations + iter + 2];
+                        pointIndicesToDecode[i * nIterations + iter + 2];
                 const auto weight2 = weights[i * nIterations + iter + 2];
                 T::accum(
                         pqFineCentroidsQ,
@@ -1377,13 +1377,13 @@ static void verifyMinMaxIndexPQDecoder(
             float outputAccumMin = 0;
             for (uint64_t iter = 0; iter < nIterations; iter += 3) {
                 const auto pointIdx0 =
-                        pointIncidesToDecode[i * nIterations + iter + 0];
+                        pointIndicesToDecode[i * nIterations + iter + 0];
                 const auto weight0 = weights[i * nIterations + iter + 0];
                 const auto pointIdx1 =
-                        pointIncidesToDecode[i * nIterations + iter + 1];
+                        pointIndicesToDecode[i * nIterations + iter + 1];
                 const auto weight1 = weights[i * nIterations + iter + 1];
                 const auto pointIdx2 =
-                        pointIncidesToDecode[i * nIterations + iter + 2];
+                        pointIndicesToDecode[i * nIterations + iter + 2];
                 const auto weight2 = weights[i * nIterations + iter + 2];
                 T::accum(
                         pqFineCentroidsQ,
