@@ -558,7 +558,7 @@ class IndexBase:
             "construction_params": self.get_construction_params(),
             "search_params": search_parameters,
         }
-        logger.info("range_seach: end")
+        logger.info("range_search: end")
         return lims, D, I, R, P, None
 
 
@@ -1138,6 +1138,8 @@ class IndexFromFactory(Index):
                 return None, None, ""
             logger.info(f"assemble, train {self.factory}")
             xt = self.io.get_dataset(self.training_vectors)
+            if self.training_vectors.normalize_L2:
+                faiss.normalize_L2(xt)
             _, t, _ = timer("train", lambda: codec.train(xt), once=True)
             t_aggregate += t
 
