@@ -10,6 +10,7 @@ set -e
 # Build libfaiss.so/libfaiss_avx2.so/libfaiss_avx512.so
 cmake -B _build \
       -DBUILD_SHARED_LIBS=ON \
+      -DFAISS_ENABLE_C_API=ON \
       -DBUILD_TESTING=OFF \
       -DFAISS_OPT_LEVEL=avx512 \
       -DFAISS_ENABLE_GPU=OFF \
@@ -18,7 +19,13 @@ cmake -B _build \
       -DCMAKE_INSTALL_LIBDIR=lib \
       -DCMAKE_BUILD_TYPE=Release .
 
-make -C _build -j$(nproc) faiss faiss_avx2 faiss_avx512
+make -C _build -j$(nproc) faiss faiss_avx2 faiss_avx512 faiss_c faiss_c_avx2 faiss_c_avx512
 
 cmake --install _build --prefix $PREFIX
 cmake --install _build --prefix _libfaiss_stage/
+
+# Copy SVS runtime if it exists
+if [ -d "_build/_deps/svs-src/lib" ]; then
+    cp -P _build/_deps/svs-src/lib/libsvs_runtime.so* $PREFIX/lib/ || true
+    cp -P _build/_deps/svs-src/lib/libsvs_runtime.so* _libfaiss_stage/lib/ || true
+fi
