@@ -9,10 +9,7 @@
 
 #include <faiss/Index2Layer.h>
 
-#include <faiss/impl/platform_macros.h>
-#include <cassert>
 #include <cinttypes>
-#include <cmath>
 #include <cstdint>
 #include <cstdio>
 
@@ -85,7 +82,7 @@ void Index2Layer::train(idx_t n, const float* x) {
 
     std::unique_ptr<const float[]> del_x(x_in == x ? nullptr : x);
 
-    std::vector<idx_t> assign(n); // assignement to coarse centroids
+    std::vector<idx_t> assign(n); // assignment to coarse centroids
     q1.quantizer->assign(n, x, assign.data());
     std::vector<float> residuals(n * d);
     for (idx_t i = 0; i < n; i++) {
@@ -93,13 +90,14 @@ void Index2Layer::train(idx_t n, const float* x) {
                 x + i * d, residuals.data() + i * d, assign[i]);
     }
 
-    if (verbose)
+    if (verbose) {
         printf("training %zdx%zd product quantizer on %" PRId64
                " vectors in %dD\n",
                pq.M,
                pq.ksub,
                n,
                d);
+    }
     pq.verbose = verbose;
     pq.train(n, residuals.data());
 
