@@ -1,5 +1,5 @@
-/**
- * Copyright (c) Facebook, Inc. and its affiliates.
+/*
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * This source code is licensed under the MIT license found in the
  * LICENSE file in the root directory of this source tree.
@@ -32,7 +32,7 @@ struct IndexBinaryFromFloat : IndexBinary {
 
     explicit IndexBinaryFromFloat(Index* index);
 
-    ~IndexBinaryFromFloat();
+    ~IndexBinaryFromFloat() override;
 
     void add(idx_t n, const uint8_t* x) override;
 
