@@ -81,6 +81,7 @@ struct PQDistanceComputer : FlatCodesDistanceComputer {
     const float* sdc;
     std::vector<float> precomputed_table;
     size_t ndis;
+    const float* q;
 
     float distance_to_code(const uint8_t* code) final {
         ndis++;
@@ -109,7 +110,8 @@ struct PQDistanceComputer : FlatCodesDistanceComputer {
             : FlatCodesDistanceComputer(
                       storage.codes.data(),
                       storage.code_size),
-              pq(storage.pq) {
+              pq(storage.pq),
+              q(nullptr) {
         precomputed_table.resize(pq.M * pq.ksub);
         nb = storage.ntotal;
         d = storage.d;
@@ -123,6 +125,7 @@ struct PQDistanceComputer : FlatCodesDistanceComputer {
     }
 
     void set_query(const float* x) override {
+        q = x;
         if (metric == METRIC_L2) {
             pq.compute_distance_table(x, precomputed_table.data());
         } else {
