@@ -31,11 +31,13 @@ void sync_d(IndexBinary* index) {
 
 // add translation to all valid labels
 void translate_labels(int64_t n, idx_t* labels, int64_t translation) {
-    if (translation == 0)
+    if (translation == 0) {
         return;
+    }
     for (int64_t i = 0; i < n; i++) {
-        if (labels[i] < 0)
+        if (labels[i] < 0) {
             continue;
+        }
         labels[i] += translation;
     }
 }
@@ -199,8 +201,6 @@ void IndexShardsTemplate<IndexT>::search(
         distance_t* distances,
         idx_t* labels,
         const SearchParameters* params) const {
-    FAISS_THROW_IF_NOT_MSG(
-            !params, "search params not supported for this index");
     FAISS_THROW_IF_NOT(k > 0);
 
     int64_t nshard = this->count();
@@ -219,7 +219,7 @@ void IndexShardsTemplate<IndexT>::search(
         }
     }
 
-    auto fn = [n, k, x, &all_distances, &all_labels, &translations](
+    auto fn = [n, k, x, params, &all_distances, &all_labels, &translations](
                       int no, const IndexT* index) {
         if (index->verbose) {
             printf("begin query shard %d on %" PRId64 " points\n", no, n);
@@ -230,7 +230,8 @@ void IndexShardsTemplate<IndexT>::search(
                 x,
                 k,
                 all_distances.data() + no * k * n,
-                all_labels.data() + no * k * n);
+                all_labels.data() + no * k * n,
+                params);
 
         translate_labels(
                 n * k, all_labels.data() + no * k * n, translations[no]);
@@ -263,7 +264,7 @@ void IndexShardsTemplate<IndexT>::search(
     }
 }
 
-// explicit instanciations
+// explicit instantiations
 template struct IndexShardsTemplate<Index>;
 template struct IndexShardsTemplate<IndexBinary>;
 
