@@ -15,6 +15,7 @@
 #include <faiss/IndexBinaryFlat.h>
 #include <faiss/IndexFlat.h>
 #if defined USE_NVIDIA_CUVS
+#include <faiss/IndexBinaryHNSW.h>
 #include <faiss/IndexHNSW.h>
 #endif
 #include <faiss/IndexIVF.h>
@@ -28,14 +29,13 @@
 #include <faiss/gpu/GpuIndex.h>
 #include <faiss/gpu/GpuIndexBinaryFlat.h>
 #if defined USE_NVIDIA_CUVS
+#include <faiss/gpu/GpuIndexBinaryCagra.h>
 #include <faiss/gpu/GpuIndexCagra.h>
 #endif
 #include <faiss/gpu/GpuIndexFlat.h>
 #include <faiss/gpu/GpuIndexIVFFlat.h>
 #include <faiss/gpu/GpuIndexIVFPQ.h>
 #include <faiss/gpu/GpuIndexIVFScalarQuantizer.h>
-#include <faiss/gpu/utils/DeviceUtils.h>
-#include <faiss/impl/FaissAssert.h>
 #include <faiss/index_io.h>
 
 namespace faiss {
@@ -95,6 +95,9 @@ Index* ToCPUCloner::clone_Index(const Index* index) {
 #if defined USE_NVIDIA_CUVS
     else if (auto icg = dynamic_cast<const GpuIndexCagra*>(index)) {
         IndexHNSWCagra* res = new IndexHNSWCagra();
+        if (icg->get_numeric_type() != faiss::NumericType::Float32) {
+            res->base_level_only = true;
+        }
         icg->copyTo(res);
         return res;
     }
@@ -236,7 +239,7 @@ Index* ToGpuCloner::clone_Index(const Index* index) {
         config.device = device;
         GpuIndexCagra* res =
                 new GpuIndexCagra(provider, icg->d, icg->metric_type, config);
-        res->copyFrom(icg);
+        res->copyFrom_ex(icg, icg->get_numeric_type());
         return res;
     }
 #endif
@@ -290,14 +293,16 @@ void ToGpuClonerMultiple::copy_ivf_shard(
         idx_t i0 = i * index_ivf->ntotal / n;
         idx_t i1 = (i + 1) * index_ivf->ntotal / n;
 
-        if (verbose)
+        if (verbose) {
             printf("IndexShards shard %ld indices %ld:%ld\n", i, i0, i1);
+        }
         index_ivf->copy_subset_to(
                 *idx2, InvertedLists::SUBSET_TYPE_ID_RANGE, i0, i1);
         FAISS_ASSERT(idx2->ntotal == i1 - i0);
     } else if (shard_type == 1) {
-        if (verbose)
+        if (verbose) {
             printf("IndexShards shard %ld select modulo %ld = %ld\n", i, n, i);
+        }
         index_ivf->copy_subset_to(
                 *idx2, InvertedLists::SUBSET_TYPE_ID_MOD, n, i);
     } else if (shard_type == 4) {
@@ -527,7 +532,15 @@ faiss::IndexBinary* index_binary_gpu_to_cpu(
         IndexBinaryFlat* ret = new IndexBinaryFlat();
         ii->copyTo(ret);
         return ret;
-    } else {
+    }
+#if defined USE_NVIDIA_CUVS
+    else if (auto ii = dynamic_cast<const GpuIndexBinaryCagra*>(gpu_index)) {
+        IndexBinaryHNSWCagra* ret = new IndexBinaryHNSWCagra();
+        ii->copyTo(ret);
+        return ret;
+    }
+#endif
+    else {
         FAISS_THROW_MSG("cannot clone this type of index");
     }
 }
@@ -540,11 +553,20 @@ faiss::IndexBinary* index_binary_cpu_to_gpu(
     if (auto ii = dynamic_cast<const IndexBinaryFlat*>(index)) {
         GpuIndexBinaryFlatConfig config;
         config.device = device;
-        if (options) {
-            config.use_cuvs = options->use_cuvs;
-        }
         return new GpuIndexBinaryFlat(provider, ii, config);
-    } else {
+    }
+#if defined USE_NVIDIA_CUVS
+    else if (
+            auto ii = dynamic_cast<const faiss::IndexBinaryHNSWCagra*>(index)) {
+        GpuIndexCagraConfig config;
+        config.device = device;
+        GpuIndexBinaryCagra* res =
+                new GpuIndexBinaryCagra(provider, ii->d, config);
+        res->copyFrom(ii);
+        return res;
+    }
+#endif
+    else {
         FAISS_THROW_MSG("cannot clone this type of index");
     }
 }
