@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -20,9 +20,11 @@ def to_column_major_torch(x):
         # was default setting before memory_format was introduced
         return x.t().clone().t()
 
+
 def to_column_major_numpy(x):
     return x.T.copy().T
 
+
 class TestTorchUtilsGPU(unittest.TestCase):
     # tests add, search
     def test_lookup(self):
@@ -80,7 +82,11 @@ class TestTorchUtilsGPU(unittest.TestCase):
         res = faiss.StandardGpuResources()
         res.noTempMemory()
 
-        index = faiss.GpuIndexIVFFlat(res, d, nlist, faiss.METRIC_L2)
+        config = faiss.GpuIndexIVFFlatConfig()
+        # FIXME: triage failure when use_cuvs is set to True (issue #3968)
+        config.use_cuvs = False
+
+        index = faiss.GpuIndexIVFFlat(res, d, nlist, faiss.METRIC_L2, config)
         xb = torch.rand(1000, d, device=torch.device('cuda', 0), dtype=torch.float32)
         index.train(xb)
 
@@ -167,7 +173,7 @@ class TestTorchUtilsGPU(unittest.TestCase):
         res = faiss.StandardGpuResources()
         res.noTempMemory()
         config = faiss.GpuIndexIVFFlatConfig()
-        config.use_raft = False
+        config.use_cuvs = False
 
         index = faiss.GpuIndexIVFFlat(res, d, nlist, faiss.METRIC_L2, config)
 
@@ -252,8 +258,9 @@ class TestTorchUtilsGPU(unittest.TestCase):
         # This is not currently implemented on GPU indices
         return
 
+
 class TestTorchUtilsKnnGpu(unittest.TestCase):
-    def test_knn_gpu(self, use_raft=False):
+    def test_knn_gpu(self, use_cuvs=False):
         torch.manual_seed(10)
         d = 32
         nb = 1024
@@ -290,7 +297,7 @@ class TestTorchUtilsKnnGpu(unittest.TestCase):
                     else:
                         xb_c = xb_np
 
-                    D, I = faiss.knn_gpu(res, xq_c, xb_c, k, use_raft=use_raft)
+                    D, I = faiss.knn_gpu(res, xq_c, xb_c, k, use_cuvs=use_cuvs)
 
                     self.assertTrue(torch.equal(torch.from_numpy(I), gt_I))
                     self.assertLess((torch.from_numpy(D) - gt_D).abs().max(), 1e-4)
@@ -316,7 +323,7 @@ class TestTorchUtilsKnnGpu(unittest.TestCase):
                             xb_c = to_column_major_torch(xb)
                             assert not xb_c.is_contiguous()
 
-                        D, I = faiss.knn_gpu(res, xq_c, xb_c, k, use_raft=use_raft)
+                        D, I = faiss.knn_gpu(res, xq_c, xb_c, k, use_cuvs=use_cuvs)
 
                         self.assertTrue(torch.equal(I.cpu(), gt_I))
                         self.assertLess((D.cpu() - gt_D).abs().max(), 1e-4)
@@ -324,7 +331,7 @@ class TestTorchUtilsKnnGpu(unittest.TestCase):
                         # test on subset
                         try:
                             # This internally uses the current pytorch stream
-                            D, I = faiss.knn_gpu(res, xq_c[6:8], xb_c, k, use_raft=use_raft)
+                            D, I = faiss.knn_gpu(res, xq_c[6:8], xb_c, k, use_cuvs=use_cuvs)
                         except TypeError:
                             if not xq_row_major:
                                 # then it is expected
@@ -336,12 +343,12 @@ class TestTorchUtilsKnnGpu(unittest.TestCase):
                         self.assertLess((D.cpu() - gt_D[6:8]).abs().max(), 1e-4)
 
     @unittest.skipUnless(
-        "RAFT" in faiss.get_compile_options(),
-        "only if RAFT is compiled in")
-    def test_knn_gpu_raft(self):
-        self.test_knn_gpu(use_raft=True)
+        "CUVS" in faiss.get_compile_options(),
+        "only if CUVS is compiled in")
+    def test_knn_gpu_cuvs(self):
+        self.test_knn_gpu(use_cuvs=True)
 
-    def test_knn_gpu_datatypes(self, use_raft=False):
+    def test_knn_gpu_datatypes(self, use_cuvs=False):
         torch.manual_seed(10)
         d = 10
         nb = 1024
@@ -364,7 +371,7 @@ class TestTorchUtilsKnnGpu(unittest.TestCase):
         D = torch.zeros(nq, k, device=xb_c.device, dtype=torch.float32)
         I = torch.zeros(nq, k, device=xb_c.device, dtype=torch.int32)
 
-        faiss.knn_gpu(res, xq_c, xb_c, k, D, I, use_raft=use_raft)
+        faiss.knn_gpu(res, xq_c, xb_c, k, D, I, use_cuvs=use_cuvs)
 
         self.assertTrue(torch.equal(I.long().cpu(), gt_I))
         self.assertLess((D.float().cpu() - gt_D).abs().max(), 1.5e-3)
@@ -376,7 +383,7 @@ class TestTorchUtilsKnnGpu(unittest.TestCase):
         xb_c = xb.half().numpy()
         xq_c = xq.half().numpy()
 
-        faiss.knn_gpu(res, xq_c, xb_c, k, D, I, use_raft=use_raft)
+        faiss.knn_gpu(res, xq_c, xb_c, k, D, I, use_cuvs=use_cuvs)
 
         self.assertTrue(torch.equal(torch.from_numpy(I).long(), gt_I))
         self.assertLess((torch.from_numpy(D) - gt_D).abs().max(), 1.5e-3)
