@@ -13,7 +13,6 @@
 #include <cstdio>
 
 #include <faiss/impl/platform_macros.h>
-#include <omp.h>
 
 #ifdef __SSE__
 #include <immintrin.h>
@@ -322,7 +321,7 @@ struct Codec6bit {
     static FAISS_ALWAYS_INLINE __m256
     decode_8_components(const uint8_t* code, int i) {
         // // Faster code for Intel CPUs or AMD Zen3+, just keeping it here
-        // // for the reference, maybe, it becomes used oned day.
+        // // for the reference, maybe, it becomes used one day.
         // const uint16_t* data16 = (const uint16_t*)(code + (i >> 2) * 3);
         // const uint32_t* data32 = (const uint32_t*)data16;
         // const uint64_t val = *data32 + ((uint64_t)data16[2] << 32);
@@ -985,10 +984,12 @@ void train_Uniform(
         vmin = HUGE_VAL;
         vmax = -HUGE_VAL;
         for (size_t i = 0; i < n; i++) {
-            if (x[i] < vmin)
+            if (x[i] < vmin) {
                 vmin = x[i];
-            if (x[i] > vmax)
+            }
+            if (x[i] > vmax) {
                 vmax = x[i];
+            }
         }
         float vexp = (vmax - vmin) * rs_arg;
         vmin -= vexp;
@@ -1008,14 +1009,13 @@ void train_Uniform(
     } else if (rs == ScalarQuantizer::RS_quantiles) {
         std::vector<float> x_copy(n);
         memcpy(x_copy.data(), x, n * sizeof(*x));
-        // TODO just do a quickselect
-        std::sort(x_copy.begin(), x_copy.end());
-        int o = int(rs_arg * n);
-        if (o < 0)
-            o = 0;
-        if (o > n - o)
-            o = n / 2;
+        int temp = int(rs_arg * n);
+        int o = temp < 0 ? 0 : (temp > n / 2 ? n / 2 : temp);
+
+        std::nth_element(x_copy.begin(), x_copy.begin() + o, x_copy.end());
         vmin = x_copy[o];
+        std::nth_element(
+                x_copy.begin(), x_copy.begin() + (n - 1 - o), x_copy.end());
         vmax = x_copy[n - 1 - o];
 
     } else if (rs == ScalarQuantizer::RS_optim) {
@@ -1024,10 +1024,12 @@ void train_Uniform(
         {
             vmin = HUGE_VAL, vmax = -HUGE_VAL;
             for (size_t i = 0; i < n; i++) {
-                if (x[i] < vmin)
+                if (x[i] < vmin) {
                     vmin = x[i];
-                if (x[i] > vmax)
+                }
+                if (x[i] > vmax) {
                     vmax = x[i];
+                }
                 sx += x[i];
             }
             b = vmin;
@@ -1043,10 +1045,12 @@ void train_Uniform(
             for (idx_t i = 0; i < n; i++) {
                 float xi = x[i];
                 float ni = floor((xi - b) / a + 0.5);
-                if (ni < 0)
+                if (ni < 0) {
                     ni = 0;
-                if (ni >= k)
+                }
+                if (ni >= k) {
                     ni = k - 1;
+                }
                 err1 += sqr(xi - (ni * a + b));
                 sn += ni;
                 sn2 += ni * ni;
@@ -1055,8 +1059,9 @@ void train_Uniform(
 
             if (err1 == last_err) {
                 iter_last_err++;
-                if (iter_last_err == 16)
+                if (iter_last_err == 16) {
                     break;
+                }
             } else {
                 last_err = err1;
                 iter_last_err = 0;
@@ -1071,8 +1076,9 @@ void train_Uniform(
                 fflush(stdout);
             }
         }
-        if (verbose)
+        if (verbose) {
             printf("\n");
+        }
 
         vmin = b;
         vmax = b + a * (k - 1);
@@ -1100,10 +1106,12 @@ void train_NonUniform(
         for (size_t i = 1; i < n; i++) {
             const float* xi = x + i * d;
             for (size_t j = 0; j < d; j++) {
-                if (xi[j] < vmin[j])
+                if (xi[j] < vmin[j]) {
                     vmin[j] = xi[j];
-                if (xi[j] > vmax[j])
+                }
+                if (xi[j] > vmax[j]) {
                     vmax[j] = xi[j];
+                }
             }
         }
         float* vdiff = vmax;
@@ -2066,16 +2074,18 @@ void ScalarQuantizer::compute_codes(const float* x, uint8_t* codes, size_t n)
 
     memset(codes, 0, code_size * n);
 #pragma omp parallel for
-    for (int64_t i = 0; i < n; i++)
+    for (int64_t i = 0; i < n; i++) {
         squant->encode_vector(x + i * d, codes + i * code_size);
+    }
 }
 
 void ScalarQuantizer::decode(const uint8_t* codes, float* x, size_t n) const {
     std::unique_ptr<SQuantizer> squant(select_quantizer());
 
 #pragma omp parallel for
-    for (int64_t i = 0; i < n; i++)
+    for (int64_t i = 0; i < n; i++) {
         squant->decode_vector(codes + i * code_size, x + i * d);
+    }
 }
 
 SQDistanceComputer* ScalarQuantizer::get_distance_computer(
