@@ -164,17 +164,16 @@ def add_ref_in_function(function_name, parameter_no):
     setattr(this_module, function_name, replacement_function)
 
 
-try:
+if "GPU" in get_compile_options():
     add_ref_in_constructor(GpuIndexIVFFlat, 1)
     add_ref_in_constructor(GpuIndexBinaryFlat, 1)
     add_ref_in_constructor(GpuIndexFlat, 1)
     add_ref_in_constructor(GpuIndexIVFPQ, 1)
     add_ref_in_constructor(GpuIndexIVFScalarQuantizer, 1)
-except NameError as e:
-    logger.info("Failed to load GPU Faiss: %s. Will not load constructor refs for GPU indexes. This is only an error if you're trying to use GPU Faiss." % e.args[0])
 
 add_ref_in_constructor(IndexIVFFlat, 0)
 add_ref_in_constructor(IndexIVFFlatDedup, 0)
+add_ref_in_constructor(IndexIVFFlatPanorama, 0)
 add_ref_in_constructor(IndexPreTransform, {2: [0, 1], 1: [0]})
 add_ref_in_method(IndexPreTransform, 'prepend_transform', 0)
 add_ref_in_constructor(IndexIVFPQ, 0)
@@ -198,6 +197,7 @@ add_ref_in_constructor(IndexHNSW, 0)
 add_ref_in_method(IndexShards, 'add_shard', 0)
 add_ref_in_method(IndexBinaryShards, 'add_shard', 0)
 add_ref_in_constructor(IndexRefineFlat, {2: [0], 1: [0]})
+add_ref_in_constructor(IndexRefinePanorama, {2: [0, 1]})
 add_ref_in_constructor(IndexRefine, {2: [0, 1]})
 
 add_ref_in_constructor(IndexBinaryIVF, 0)
@@ -220,6 +220,15 @@ add_ref_in_constructor(IDSelectorTranslated, slice(2))
 add_ref_in_constructor(IDSelectorXOr, slice(2))
 add_ref_in_constructor(IndexIVFIndependentQuantizer, slice(3))
 
+add_ref_in_constructor(IndexIVFRaBitQ, 0)
+add_ref_in_constructor(IndexIVFRaBitQFastScan, 0)
+
+if "SVS" in get_compile_options():
+    add_ref_in_constructor(IndexSVSVamana, 0)
+    add_ref_in_constructor(IndexSVSVamanaLVQ, 0)
+    add_ref_in_constructor(IndexSVSVamanaLeanVec, 0)
+    add_ref_in_constructor(IndexSVSFlat, 0)
+
 # seems really marginal...
 # remove_ref_from_method(IndexReplicas, 'removeIndex', 0)
 
