@@ -3,8 +3,6 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-import inspect
-
 import faiss
 import numpy as np
 
@@ -36,12 +34,14 @@ from faiss.loader import (
 # because it is unclear how the conversion should occur: with a view
 # (= cast) or conversion?
 
+
 def _check_dtype_uint8(codes):
     if codes.dtype != 'uint8':
         raise TypeError("Input argument %s must be ndarray of dtype "
                         " uint8, but found %s" % ("codes", codes.dtype))
     return np.ascontiguousarray(codes)
 
+
 def _numeric_to_str(numeric_type):
     if numeric_type == faiss.Float32:
         return 'float32'
@@ -52,6 +52,7 @@ def _numeric_to_str(numeric_type):
     else:
         raise ValueError("numeric type must be either faiss.Float32, faiss.Float16, or faiss.Int8")
 
+
 def replace_method(the_class, name, replacement, ignore_missing=False):
     """ Replaces a method in a class with another version. The old method
     is renamed to method_name_c (because presumably it was implemented in C) """
@@ -220,7 +221,40 @@ def handle_NSG(the_class):
 
 def handle_Index(the_class):
 
-    def replacement_add(self, x, numeric_type = faiss.Float32):
+    def replacement_setattr(self, name, value):
+        # Prevent silent failures when setting attributes that don't exist
+        # as described in GitHub issue 3766
+
+        # Allow SWIG internal attributes that are essential for object
+        # functionality
+        if name in ['this', 'thisown']:
+            return original_setattr(self, name, value)
+
+        # Allow internal Faiss attributes used during construction/operation
+        if name in ['referenced_objects']:
+            return original_setattr(self, name, value)
+
+        # Check if the attribute already exists (valid attribute)
+        try:
+            # Check if it exists on the instance or class
+            if hasattr(self, name) or hasattr(self.__class__, name):
+                return original_setattr(self, name, value)
+        except (AttributeError, TypeError, SystemError):
+            # During object construction, hasattr might fail, so be permissive
+            return original_setattr(self, name, value)
+
+        # If we reach here, the attribute doesn't exist on the object
+        # This is the core issue: SWIG classes silently accept unknown
+        # attributes
+        # We should generally block this to prevent silent failures
+
+        # Block unknown attributes to prevent silent failures
+        # This is the general solution that doesn't rely on hardcoded names
+        raise AttributeError(
+            f"'{self.__class__.__name__}' object has no attribute '{name}'."
+        )
+
+    def replacement_add(self, x, numeric_type=faiss.Float32):
         """Adds vectors to the index.
         The index must be trained before vectors can be added to it.
         The vectors are implicitly numbered in sequence. When `n` vectors are
@@ -239,7 +273,7 @@ def handle_Index(the_class):
         if numeric_type == faiss.Float32:
             self.add_c(n, swig_ptr(x))
         else:
-            self.addEx(n, swig_ptr(x), numeric_type)
+            self.add_ex(n, swig_ptr(x), numeric_type)
 
     def replacement_add_with_ids(self, x, ids, numeric_type = faiss.Float32):
         """Adds vectors with arbitrary ids to the index (not all indexes support this).
@@ -263,7 +297,7 @@ def handle_Index(the_class):
         if numeric_type == faiss.Float32:
             self.add_with_ids_c(n, swig_ptr(x), swig_ptr(ids))
         else:
-            self.add_with_idsEx(n, swig_ptr(x), numeric_type, swig_ptr(ids))
+            self.add_with_ids_ex(n, swig_ptr(x), numeric_type, swig_ptr(ids))
 
 
     def replacement_assign(self, x, k, labels=None):
@@ -314,8 +348,8 @@ def handle_Index(the_class):
         if numeric_type == faiss.Float32:
             self.train_c(n, swig_ptr(x))
         else:
-            self.trainEx(n, swig_ptr(x), numeric_type)
-        
+            self.train_ex(n, swig_ptr(x), numeric_type)
+
 
     def replacement_search(self, x, k, *, params=None, D=None, I=None, numeric_type = faiss.Float32):
         """Find the k nearest neighbors of the set of vectors x in the index.
@@ -363,7 +397,7 @@ def handle_Index(the_class):
         if numeric_type == faiss.Float32:
             self.search_c(n, swig_ptr(x), k, swig_ptr(D), swig_ptr(I), params)
         else:
-            self.searchEx(n, swig_ptr(x), numeric_type, k, swig_ptr(D), swig_ptr(I), params)
+            self.search_ex(n, swig_ptr(x), numeric_type, k, swig_ptr(D), swig_ptr(I), params)
         return D, I
 
     def replacement_search_and_reconstruct(self, x, k, *, params=None, D=None, I=None, R=None):
@@ -839,6 +873,13 @@ def handle_Index(the_class):
     replace_method(the_class, 'permute_entries', replacement_permute_entries,
                    ignore_missing=True)
 
+    # Store the original __setattr__ method
+    original_setattr = (the_class.__setattr__ if
+                        hasattr(the_class, '__setattr__')
+                        else object.__setattr__)
+
+    the_class.__setattr__ = replacement_setattr
+
     # get/set state for pickle
     # the data is serialized to std::vector -> numpy array -> python bytes
     # so not very efficient for now.
@@ -1052,7 +1093,7 @@ def handle_VectorTransform(the_class):
         self.train_c(n, swig_ptr(x))
 
     replace_method(the_class, 'train', replacement_vt_train)
-    # apply is reserved in Pyton...
+    # apply is reserved in Python...
     the_class.apply_py = apply_method
     the_class.apply = apply_method
     replace_method(the_class, 'reverse_transform',
@@ -1123,7 +1164,7 @@ def handle_IndexRowwiseMinMax(the_class):
         The index must be trained before vectors can be added to it.
 
         This call WILL change the values in the input array, because
-        of two scaling proceduces being performed inplace.
+        of two scaling procedures being performed inplace.
 
         Parameters
         ----------
@@ -1193,6 +1234,7 @@ def add_to_referenced_objects(self, ref):
     else:
         self.referenced_objects.append(ref)
 
+
 class RememberSwigOwnership:
     """
     SWIG's seattr transfers ownership of SWIG wrapped objects to the class
@@ -1220,7 +1262,7 @@ def handle_SearchParameters(the_class):
     """ this wrapper is to enable initializations of the form
     SearchParametersXX(a=3, b=SearchParamsYY)
     This also requires the enclosing class to keep a reference on the
-    sub-object, since the C++ code assumes the object ownwership is
+    sub-object, since the C++ code assumes the object ownership is
     handled externally.
     """
     the_class.original_init = the_class.__init__
@@ -1272,7 +1314,7 @@ def handle_CodeSet(the_class):
     replace_method(the_class, 'insert', replacement_insert)
 
 ######################################################
-# Syntatic sugar for NeuralNet classes
+# Syntactic sugar for NeuralNet classes
 ######################################################
 
 
@@ -1363,9 +1405,10 @@ def handle_Linear(the_class):
     the_class.from_torch = from_torch
 
 ######################################################
-# Syntatic sugar for QINCo and QINCoStep
+# Syntactic sugar for QINCo and QINCoStep
 ######################################################
 
+
 def handle_QINCoStep(the_class):
     the_class.original_init = the_class.__init__
 
