@@ -7,9 +7,9 @@
 
 #include <faiss/utils/distances_fused/distances_fused.h>
 
-#include <faiss/impl/platform_macros.h>
+#include <faiss/impl/platform_macros.h> // NOLINT
 
-#include <faiss/utils/distances_fused/avx512.h>
+#include <faiss/utils/distances_fused/avx512.h> // NOLINT
 #include <faiss/utils/distances_fused/simdlib_based.h>
 
 namespace faiss {
