@@ -8,7 +8,7 @@
 // -*- c++ -*-
 
 /*
- *  A few utilitary functions for similarity search:
+ *  A few utility functions for similarity search:
  * - optimized exhaustive distance and knn search functions
  * - some functions reimplemented from torch for speed
  */
@@ -64,7 +64,7 @@ void matrix_qr(int m, int n, float* a);
 void ranklist_handle_ties(int k, int64_t* idx, const float* dis);
 
 /** count the number of common elements between v1 and v2
- * algorithm = sorting + bissection to avoid double-counting duplicates
+ * algorithm = sorting + bisection to avoid double-counting duplicates
  */
 size_t ranklist_intersection_size(
         size_t k1,
@@ -92,10 +92,10 @@ size_t merge_result_table_with(
 
 /// a balanced assignment has a IF of 1, a completely unbalanced assignment has
 /// an IF = k.
-double imbalance_factor(int n, int k, const int64_t* assign);
+double imbalance_factor(int64_t n, int k, const int64_t* assign);
 
 /// same, takes a histogram as input
-double imbalance_factor(int k, const int* hist);
+double imbalance_factor(int k, const int64_t* hist);
 
 /// compute histogram on v
 int ivec_hist(size_t n, const int* v, int vmax, int* hist);
