@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -80,7 +80,7 @@ class TestMapLong2Long(unittest.TestCase):
         assert m.search(12343) == -1
 
 
-class TestOrthognalReconstruct(unittest.TestCase):
+class TestOrthogonalReconstruct(unittest.TestCase):
 
     def test_recons_orthonormal(self):
         lt = faiss.LinearTransform(20, 10, True)
@@ -103,7 +103,7 @@ class TestOrthognalReconstruct(unittest.TestCase):
 
         self.assertGreater(1e-5, err)
 
-    def test_recons_orthogona_impossible(self):
+    def test_recons_orthogonal_impossible(self):
         lt = faiss.LinearTransform(20, 10, True)
         rs = np.random.RandomState(10)
         A = rs.randn(10 * 20).astype('float32')
