@@ -70,12 +70,13 @@ std::vector<idx_t> search_index(Index* index, const float* xq) {
 struct EncapsulateInvertedLists : InvertedLists {
     const InvertedLists* il;
 
-    EncapsulateInvertedLists(const InvertedLists* il)
+    explicit EncapsulateInvertedLists(const InvertedLists* il)
             : InvertedLists(il->nlist, il->code_size), il(il) {}
 
     static void* memdup(const void* m, size_t size) {
-        if (size == 0)
+        if (size == 0) {
             return nullptr;
+        }
         return memcpy(malloc(size), m, size);
     }
 
