@@ -6,8 +6,6 @@
 
 import unittest
 import time
-import os
-import tempfile
 
 import numpy as np
 import faiss
@@ -17,6 +15,7 @@ from faiss.contrib import datasets
 # the tests tend to timeout in stress modes + dev otherwise
 faiss.omp_set_num_threads(4)
 
+
 class TestSearch(unittest.TestCase):
 
     def test_PQ4_accuracy(self):
@@ -44,6 +43,7 @@ class TestSearch(unittest.TestCase):
     # not exploitable, hence the flag test on that as well.
     @unittest.skipUnless(
         ('AVX2' in faiss.get_compile_options() or
+        'AVX512' in faiss.get_compile_options() or
         'NEON' in faiss.get_compile_options()) and
         "OPTIMIZE" in faiss.get_compile_options(),
         "only test while building with avx2 or neon")
@@ -182,6 +182,8 @@ def reference_accu(codes, LUT):
 
 
 # disabled because the function to write to mem is not implemented currently
+
+
 class ThisIsNotATestLoop5:    # (unittest.TestCase):
 
     def do_loop5_kernel(self, nq, bb):
@@ -519,7 +521,6 @@ class TestAQFastScan(unittest.TestCase):
         index = faiss.index_factory(d, 'RQ8x4' + st, metric_type)
         index.train(ds.get_train())
         index.add(ds.get_database())
-        index.nprobe = 16
         Dref, Iref = index.search(ds.get_queries(), 1)
 
         indexfs = faiss.IndexAdditiveQuantizerFastScan(index)
@@ -584,16 +585,9 @@ class TestAQFastScan(unittest.TestCase):
         index.add(ds.get_database())
         D1, I1 = index.search(ds.get_queries(), 1)
 
-        fd, fname = tempfile.mkstemp()
-        os.close(fd)
-        try:
-            faiss.write_index(index, fname)
-            index2 = faiss.read_index(fname)
-            D2, I2 = index2.search(ds.get_queries(), 1)
-            np.testing.assert_array_equal(I1, I2)
-        finally:
-            if os.path.exists(fname):
-                os.unlink(fname)
+        index2 = faiss.deserialize_index(faiss.serialize_index(index))
+        D2, I2 = index2.search(ds.get_queries(), 1)
+        np.testing.assert_array_equal(I1, I2)
 
     def test_io(self):
         self.subtest_io('LSQ4x4fs_Nlsq2x4')
@@ -682,16 +676,9 @@ class TestPAQFastScan(unittest.TestCase):
         index.add(ds.get_database())
         D1, I1 = index.search(ds.get_queries(), 1)
 
-        fd, fname = tempfile.mkstemp()
-        os.close(fd)
-        try:
-            faiss.write_index(index, fname)
-            index2 = faiss.read_index(fname)
-            D2, I2 = index2.search(ds.get_queries(), 1)
-            np.testing.assert_array_equal(I1, I2)
-        finally:
-            if os.path.exists(fname):
-                os.unlink(fname)
+        index2 = faiss.deserialize_index(faiss.serialize_index(index))
+        D2, I2 = index2.search(ds.get_queries(), 1)
+        np.testing.assert_array_equal(I1, I2)
 
     def test_io(self):
         self.subtest_io('PLSQ2x3x4fs_Nlsq2x4')
