@@ -4,9 +4,7 @@
 # LICENSE file in the root directory of this source tree.
 
 
-import os
 import unittest
-import tempfile
 
 import numpy as np
 import faiss
@@ -17,6 +15,7 @@ from faiss.contrib.inspect_tools import get_invlist
 # the tests tend to timeout in stress modes + dev otherwise
 faiss.omp_set_num_threads(4)
 
+
 class TestLUTQuantization(unittest.TestCase):
 
     def compute_dis_float(self, codes, LUT, bias):
@@ -150,6 +149,7 @@ def verify_with_draws(testcase, Dref, Iref, Dnew, Inew):
             mask = Dref[i, :] == dis
             testcase.assertEqual(set(Iref[i, mask]), set(Inew[i, mask]))
 
+
 def three_metrics(Dref, Iref, Dnew, Inew):
     nq = Iref.shape[0]
     recall_at_1 = (Iref[:, 0] == Inew[:, 0]).sum() / nq
@@ -165,6 +165,7 @@ def three_metrics(Dref, Iref, Dnew, Inew):
 # Tests for various IndexIVFPQFastScan implementations
 ##########################################################
 
+
 class TestIVFImplem1(unittest.TestCase):
     """ Verify implem 1 (search from original invlists)
     against IndexIVFPQ """
@@ -390,7 +391,7 @@ class TestIVFImplem12(unittest.TestCase):
     def test_by_residual_odd_dim(self):
         self.do_test(True, d=30)
 
-    # testin single query
+    # testing single query
     def test_no_residual_single_query(self):
         self.do_test(False, nq=1)
 
@@ -545,7 +546,7 @@ class TestTraining(unittest.TestCase):
 
 
 class TestReconstruct(unittest.TestCase):
-    """ test reconstruct and sa_encode / sa_decode 
+    """ test reconstruct and sa_encode / sa_decode
     (also for a few additive quantizer variants) """
 
     def do_test(self, by_residual=False):
@@ -574,7 +575,7 @@ class TestReconstruct(unittest.TestCase):
         index.reconstruct_orig_invlists()
         assert index.orig_invlists.compute_ntotal() == index.ntotal
 
-        # compare with non fast-scan index 
+        # compare with non fast-scan index
         index2 = faiss.IndexIVFPQ(
             index.quantizer, d, 50, d // 2, 4, metric)
         index2.by_residual = by_residual
@@ -591,11 +592,11 @@ class TestReconstruct(unittest.TestCase):
     def test_by_residual(self):
         self.do_test(by_residual=True)
 
-    def do_test_generic(self, factory_string, 
-                        by_residual=False, metric=faiss.METRIC_L2): 
+    def do_test_generic(self, factory_string,
+                        by_residual=False, metric=faiss.METRIC_L2):
         d = 32
         ds = datasets.SyntheticDataset(d, 250, 200, 10)
-        index = faiss.index_factory(ds.d, factory_string, metric) 
+        index = faiss.index_factory(ds.d, factory_string, metric)
         if "IVF" in factory_string:
             index.by_residual = by_residual
             index.make_direct_map(True)
@@ -615,7 +616,7 @@ class TestReconstruct(unittest.TestCase):
         index2 = faiss.deserialize_index(faiss.serialize_index(index))
         codes2 = index2.sa_encode(ds.get_database()[120:130])
         np.testing.assert_array_equal(codes, codes2)
-        
+
 
     def test_ivfpq_residual(self):
         self.do_test_generic("IVF20,PQ16x4fs", by_residual=True)
@@ -818,18 +819,11 @@ class TestIVFAQFastScan(unittest.TestCase):
         index = faiss.index_factory(d, factory_str)
         index.train(ds.get_train())
         index.add(ds.get_database())
-        D1, I1 = index.search(ds.get_queries(), 1)
+        _, I1 = index.search(ds.get_queries(), 1)
 
-        fd, fname = tempfile.mkstemp()
-        os.close(fd)
-        try:
-            faiss.write_index(index, fname)
-            index2 = faiss.read_index(fname)
-            D2, I2 = index2.search(ds.get_queries(), 1)
-            np.testing.assert_array_equal(I1, I2)
-        finally:
-            if os.path.exists(fname):
-                os.unlink(fname)
+        index2 = faiss.deserialize_index(faiss.serialize_index(index))
+        _, I2 = index2.search(ds.get_queries(), 1)
+        np.testing.assert_array_equal(I1, I2)
 
     def test_io(self):
         self.subtest_io('IVF16,LSQ4x4fs_Nlsq2x4')
@@ -926,16 +920,9 @@ class TestIVFPAQFastScan(unittest.TestCase):
         index.add(ds.get_database())
         D1, I1 = index.search(ds.get_queries(), 1)
 
-        fd, fname = tempfile.mkstemp()
-        os.close(fd)
-        try:
-            faiss.write_index(index, fname)
-            index2 = faiss.read_index(fname)
-            D2, I2 = index2.search(ds.get_queries(), 1)
-            np.testing.assert_array_equal(I1, I2)
-        finally:
-            if os.path.exists(fname):
-                os.unlink(fname)
+        index2 = faiss.deserialize_index(faiss.serialize_index(index))
+        D2, I2 = index2.search(ds.get_queries(), 1)
+        np.testing.assert_array_equal(I1, I2)
 
     def test_io(self):
         self.subtest_io('IVF16,PLSQ2x3x4fsr_Nlsq2x4')
