@@ -8,8 +8,6 @@
 import numpy as np
 import unittest
 import faiss
-import tempfile
-import os
 
 from common_faiss_tests import get_dataset_2
 
@@ -52,13 +50,13 @@ class TestHNSW(unittest.TestCase):
         lims, D, I = index.range_search(self.xq, radius)
 
         nmiss = 0
-        # check if returned resutls are a subset of the reference results
+        # check if returned results are a subset of the reference results
         for i in range(len(self.xq)):
             ref = Iref[lims_ref[i]: lims_ref[i + 1]]
             new = I[lims[i]: lims[i + 1]]
             self.assertLessEqual(set(new), set(ref))
             nmiss += len(ref) - len(new)
-        # currenly we miss 405 / 6019 neighbors
+        # currently we miss 405 / 6019 neighbors
         self.assertLessEqual(nmiss, lims_ref[-1] * 0.1)
 
     def test_hnsw_unbounded_queue(self):
@@ -66,7 +64,7 @@ class TestHNSW(unittest.TestCase):
 
         index = faiss.IndexHNSWFlat(d, 16)
         index.add(self.xb)
-        index.search_bounded_queue = False
+        index.hnsw.search_bounded_queue = False
         Dhnsw, Ihnsw = index.search(self.xq, 1)
 
         self.assertGreaterEqual((self.Iref == Ihnsw).sum(), 460)
@@ -201,21 +199,6 @@ class TestHNSW(unittest.TestCase):
             )
             self.assertEqual(index3.storage, None)
 
-    def test_abs_inner_product(self):
-        """Test HNSW with abs inner product (not a real distance, so dubious that triangular inequality works)"""
-        d = self.xq.shape[1]
-        xb = self.xb - self.xb.mean(axis=0)  # need to be centered to give interesting directions
-        xq = self.xq - self.xq.mean(axis=0)
-        Dref, Iref = faiss.knn(xq, xb, 10, faiss.METRIC_ABS_INNER_PRODUCT)
-
-        index = faiss.IndexHNSWFlat(d, 32, faiss.METRIC_ABS_INNER_PRODUCT)
-        index.add(xb)
-        Dnew, Inew = index.search(xq, 10)
-
-        inter = faiss.eval_intersection(Iref, Inew)
-        # 4769 vs. 500*10
-        self.assertGreater(inter, Iref.size * 0.9)
-
     def test_hnsw_reset(self):
         d = self.xb.shape[1]
         index_flat = faiss.IndexFlat(d)
@@ -232,6 +215,7 @@ class TestHNSW(unittest.TestCase):
         self.assertEqual(index_flat.ntotal, 0)
         self.assertEqual(index_hnsw.ntotal, 0)
 
+
 class Issue3684(unittest.TestCase):
 
     def test_issue3684(self):
@@ -250,11 +234,11 @@ class Issue3684(unittest.TestCase):
         hnsw_index_ip.hnsw.efSearch = 512
         hnsw_index_ip.add(xb)
 
-        # test knn 
+        # test knn
         D, I = hnsw_index_ip.search(xq, 10)
         self.assertTrue(np.all(D[:, :-1] >= D[:, 1:]))
 
-        # test range search 
+        # test range search
         radius = 0.74  # Cosine similarity threshold
         lims, D, I = hnsw_index_ip.range_search(xq, radius)
         self.assertTrue(np.all(D >= radius))
@@ -294,14 +278,7 @@ class TestNSG(unittest.TestCase):
         return knn_graph
 
     def subtest_io_and_clone(self, index, Dnsg, Insg):
-        fd, tmpfile = tempfile.mkstemp()
-        os.close(fd)
-        try:
-            faiss.write_index(index, tmpfile)
-            index2 = faiss.read_index(tmpfile)
-        finally:
-            if os.path.exists(tmpfile):
-                os.unlink(tmpfile)
+        index2 = faiss.deserialize_index(faiss.serialize_index(index))
 
         Dnsg2, Insg2 = index2.search(self.xq, 1)
         np.testing.assert_array_equal(Dnsg2, Dnsg)
@@ -320,8 +297,6 @@ class TestNSG(unittest.TestCase):
 
     def subtest_add(self, build_type, thresh, metric=faiss.METRIC_L2):
         d = self.xq.shape[1]
-        metrics = {faiss.METRIC_L2: 'L2',
-                   faiss.METRIC_INNER_PRODUCT: 'IP'}
 
         flat_index = faiss.IndexFlat(d, metric)
         flat_index.add(self.xb)
@@ -395,8 +370,6 @@ class TestNSG(unittest.TestCase):
     def test_reset(self):
         """test IndexNSG.reset()"""
         d = self.xq.shape[1]
-        metrics = {faiss.METRIC_L2: 'L2',
-                   faiss.METRIC_INNER_PRODUCT: 'IP'}
 
         metric = faiss.METRIC_L2
         flat_index = faiss.IndexFlat(d, metric)
@@ -437,6 +410,48 @@ class TestNSG(unittest.TestCase):
         gt = np.repeat(gt, nq, axis=0)  # [nq, k]
         np.testing.assert_array_equal(indices, gt)
 
+    def test_nsg_supports_pre_built_knn_graph(self):
+        """Test IndexNSGBuild"""
+        knn_graph = self.make_knn_graph(faiss.METRIC_L2)
+        d = self.xq.shape[1]
+        index = faiss.IndexNSGFlat(d, 16)
+        index.build(self.xb, knn_graph)
+        index.search(self.xq, k=1)
+        self.assertTrue(index.is_built)
+
+    def test_nsg_with_pre_built_knn_graph_throws_when_rebuilding_via_add(self):
+        """Test IndexNSGBuild"""
+        knn_graph = self.make_knn_graph(faiss.METRIC_L2)
+
+        d = self.xq.shape[1]
+        index = faiss.IndexNSGFlat(d, 16)
+        index.build(self.xb, knn_graph)
+        index.search(self.xq, k=1)
+        self.assertTrue(index.is_built)
+
+        index.GK = 32
+        index.train(self.xb)
+        with self.assertRaises(RuntimeError) as context:
+            index.add(self.xb)
+
+        self.assertIn(
+            "NSG does not support incremental addition",
+            str(context.exception)
+        )
+
+    def test_nsg_rebuild_throws_with_pre_built_knn_graph(self):
+        """Test IndexNSGBuild"""
+        knn_graph = self.make_knn_graph(faiss.METRIC_L2)
+        d = self.xq.shape[1]
+        index = faiss.IndexNSGFlat(d, 16)
+        index.build(self.xb, knn_graph)
+        index.search(self.xq, k=1)
+
+        with self.assertRaises(RuntimeError) as context:
+            index.build(self.xb, knn_graph)
+
+        self.assertIn("The IndexNSG is already built", str(context.exception))
+
     def test_nsg_pq(self):
         """Test IndexNSGPQ"""
         d = self.xq.shape[1]
@@ -518,14 +533,7 @@ class TestNNDescent(unittest.TestCase):
         self.assertGreaterEqual(recalls, 450)  # 462
 
         # do some IO tests
-        fd, tmpfile = tempfile.mkstemp()
-        os.close(fd)
-        try:
-            faiss.write_index(index, tmpfile)
-            index2 = faiss.read_index(tmpfile)
-        finally:
-            if os.path.exists(tmpfile):
-                os.unlink(tmpfile)
+        index2 = faiss.deserialize_index(faiss.serialize_index(index))
 
         D2, I2 = index2.search(self.xq, 1)
         np.testing.assert_array_equal(D2, D)
@@ -564,10 +572,6 @@ class TestNNDescentKNNG(unittest.TestCase):
         self.subtest(32, 10, faiss.METRIC_INNER_PRODUCT)
 
     def subtest(self, d, K, metric):
-        metric_names = {faiss.METRIC_L1: 'L1',
-                        faiss.METRIC_L2: 'L2',
-                        faiss.METRIC_INNER_PRODUCT: 'IP'}
-
         nb = 1000
         _, xb, _ = get_dataset_2(d, 0, nb, 0)
 
