@@ -4,7 +4,6 @@
 # LICENSE file in the root directory of this source tree.
 
 """ more elaborate that test_index.py """
-from __future__ import absolute_import, division, print_function
 
 import numpy as np
 import unittest
@@ -19,6 +18,7 @@ from faiss.contrib.datasets import SyntheticDataset
 from faiss.contrib.inspect_tools import make_LinearTransform_matrix
 from faiss.contrib.evaluation import check_ref_knn_with_draws
 
+
 class TestRemoveFastScan(unittest.TestCase):
     def do_test(self, ntotal, removed):
         d = 20
@@ -192,13 +192,7 @@ class TestRemove(unittest.TestCase):
             assert False, 'should have raised an exception'
 
         # while we are there, let's test I/O as well...
-        fd, tmpnam = tempfile.mkstemp()
-        os.close(fd)
-        try:
-            faiss.write_index_binary(index, tmpnam)
-            index = faiss.read_index_binary(tmpnam)
-        finally:
-            os.remove(tmpnam)
+        index = faiss.deserialize_index_binary(faiss.serialize_index_binary(index))
 
         assert index.reconstruct(1004)[0] == 104
         try:
@@ -371,6 +365,8 @@ class TestTransformChain(unittest.TestCase):
 
 @unittest.skipIf(platform.system() == 'Windows', \
                  'Mmap not supported on Windows.')
+
+
 class TestRareIO(unittest.TestCase):
 
     def compare_results(self, index1, index2, xq):
@@ -466,14 +462,7 @@ class TestIVFFlatDedup(unittest.TestCase):
         check_ref_knn_with_draws(Dref, Iref, Dnew, Inew)
 
         # test I/O
-        fd, tmpfile = tempfile.mkstemp()
-        os.close(fd)
-        try:
-            faiss.write_index(index_new, tmpfile)
-            index_st = faiss.read_index(tmpfile)
-        finally:
-            if os.path.exists(tmpfile):
-                os.unlink(tmpfile)
+        index_st = faiss.deserialize_index(faiss.serialize_index(index_new))
         Dst, Ist = index_st.search(xq, 20)
 
         check_ref_knn_with_draws(Dnew, Inew, Dst, Ist)
@@ -530,6 +519,8 @@ class TestSerialize(unittest.TestCase):
 
 @unittest.skipIf(platform.system() == 'Windows',
                  'OnDiskInvertedLists is unsupported on Windows.')
+
+
 class TestRenameOndisk(unittest.TestCase):
 
     def test_rename(self):
