@@ -21,6 +21,7 @@ nt = 2000
 nb = 1000
 nq = 200
 
+
 class TestIOVariants(unittest.TestCase):
 
     def test_io_error(self):
@@ -192,7 +193,6 @@ class TestCallbacks(unittest.TestCase):
             if os.path.exists(fname):
                 os.unlink(fname)
 
-
     def test_transfer_pipe(self):
         """ transfer an index through a Unix pipe """
 
@@ -359,21 +359,19 @@ class Test_IO_PQ(unittest.TestCase):
 
         try:
             faiss.write_ProductQuantizer(index.pq, fname)
-
             read_pq = faiss.read_ProductQuantizer(fname)
-
-            self.assertEqual(index.pq.M, read_pq.M)
-            self.assertEqual(index.pq.nbits, read_pq.nbits)
-            self.assertEqual(index.pq.dsub, read_pq.dsub)
-            self.assertEqual(index.pq.ksub, read_pq.ksub)
-            np.testing.assert_array_equal(
-                faiss.vector_to_array(index.pq.centroids),
-                faiss.vector_to_array(read_pq.centroids)
-            )
-
         finally:
             if os.path.exists(fname):
                 os.unlink(fname)
+        self.assertEqual(index.pq.M, read_pq.M)
+        self.assertEqual(index.pq.nbits, read_pq.nbits)
+        self.assertEqual(index.pq.dsub, read_pq.dsub)
+        self.assertEqual(index.pq.ksub, read_pq.ksub)
+        np.testing.assert_array_equal(
+            faiss.vector_to_array(index.pq.centroids),
+            faiss.vector_to_array(read_pq.centroids)
+        )
+
 
 
 class Test_IO_IndexLSH(unittest.TestCase):
@@ -452,6 +450,7 @@ class Test_IO_IndexIVFSpectralHash(unittest.TestCase):
             if os.path.exists(fname):
                 os.unlink(fname)
 
+
 class TestIVFPQRead(unittest.TestCase):
     def test_reader(self):
         d, n = 32, 1000
@@ -468,7 +467,8 @@ class TestIVFPQRead(unittest.TestCase):
             faiss.write_index(index, fname)
 
             index_a = faiss.read_index(fname)
-            index_b = faiss.read_index(fname, faiss.IO_FLAG_SKIP_PRECOMPUTE_TABLE)
+            index_b = faiss.read_index(
+                fname, faiss.IO_FLAG_SKIP_PRECOMPUTE_TABLE)
 
             Da, Ia = index_a.search(xq, 10)
             Db, Ib = index_b.search(xq, 10)
@@ -484,7 +484,6 @@ class TestIVFPQRead(unittest.TestCase):
                 os.unlink(fname)
 
 
-
 class TestIOFlatMMap(unittest.TestCase):
     @unittest.skipIf(
         platform.system() not in ["Windows", "Linux"],
@@ -519,15 +518,16 @@ class TestIOFlatMMap(unittest.TestCase):
                 except:
                     pass
 
-    def test_zerocopy(self): 
+    def test_zerocopy(self):
         xt, xb, xq = get_dataset_2(32, 0, 100, 50)
         index = faiss.index_factory(32, "SQfp16", faiss.METRIC_L2)
-        # does not need training 
+        # does not need training
         index.add(xb)
         Dref, Iref = index.search(xq, 10)
 
         serialized_index = faiss.serialize_index(index)
-        reader = faiss.ZeroCopyIOReader(faiss.swig_ptr(serialized_index), serialized_index.size)
+        reader = faiss.ZeroCopyIOReader(
+            faiss.swig_ptr(serialized_index), serialized_index.size)
         index2 = faiss.read_index(reader)
         Dnew, Inew = index2.search(xq, 10)
         np.testing.assert_array_equal(Iref, Inew)
