@@ -5,7 +5,6 @@
  * LICENSE file in the root directory of this source tree.
  */
 
-#include <cinttypes>
 #include <cstdio>
 #include <cstdlib>
 
@@ -20,6 +19,7 @@
 #include <faiss/IVFlib.h>
 #include <faiss/IndexBinaryIVF.h>
 #include <faiss/IndexIVF.h>
+#include <faiss/IndexIVFPQFastScan.h>
 #include <faiss/IndexPreTransform.h>
 #include <faiss/index_factory.h>
 #include <faiss/utils/distances.h>
@@ -64,17 +64,31 @@ std::unique_ptr<Index> make_trained_index(
     return index;
 }
 
-std::vector<idx_t> search_index(Index* index, const float* xq) {
+std::pair<std::vector<idx_t>, std::vector<float>> search_index(
+        Index* index,
+        const float* xq) {
     std::vector<idx_t> I(k * nq);
     std::vector<float> D(k * nq);
     index->search(nq, xq, k, D.data(), I.data());
-    return I;
+    return {I, D};
 }
 
 /*************************************************************
  * Test functions for a given index type
  *************************************************************/
 
+void test_get_InvertedListScanner(
+        IndexIVF* index_ivf,
+        const IndexPreTransform* index_pt,
+        std::vector<uint8_t> codes,
+        std::unique_ptr<InvertedListScanner> scanner,
+        std::vector<float> xq,
+        std::vector<idx_t> ref_I,
+        std::vector<float> ref_D,
+        MetricType metric,
+        bool distance_to_code_supported = true,
+        float accuracy_requirement = 1.0);
+
 void test_lowlevel_access(const char* index_key, MetricType metric) {
     std::unique_ptr<Index> index = make_trained_index(index_key, metric);
 
@@ -152,8 +166,30 @@ void test_lowlevel_access(const char* index_key, MetricType metric) {
 
     // sample some example queries and get reference search results.
     auto xq = make_data(nq);
-    auto ref_I = search_index(index.get(), xq.data());
+    auto res = search_index(index.get(), xq.data());
 
+    test_get_InvertedListScanner(
+            index_ivf,
+            index_pt,
+            codes,
+            nullptr,
+            xq,
+            res.first,
+            res.second,
+            metric);
+}
+
+void test_get_InvertedListScanner(
+        IndexIVF* index_ivf,
+        const IndexPreTransform* index_pt,
+        std::vector<uint8_t> codes,
+        std::unique_ptr<InvertedListScanner> scanner,
+        std::vector<float> xq,
+        std::vector<idx_t> ref_I,
+        std::vector<float> ref_D,
+        MetricType metric,
+        bool distance_to_code_supported,
+        float accuracy_requirement) {
     // handle preprocessing
     const float* xqt = xq.data();
     std::unique_ptr<float[]> del_xqt;
@@ -167,22 +203,27 @@ void test_lowlevel_access(const char* index_key, MetricType metric) {
 
     // quantize the queries to get the inverted list ids to visit.
     int nprobe = index_ivf->nprobe;
+    const InvertedLists* il = index_ivf->invlists;
 
     std::vector<idx_t> q_lists(nq * nprobe);
     std::vector<float> q_dis(nq * nprobe);
 
     index_ivf->quantizer->search(nq, xqt, nprobe, q_dis.data(), q_lists.data());
 
-    // object that does the scanning and distance computations.
-    std::unique_ptr<InvertedListScanner> scanner(
-            index_ivf->get_InvertedListScanner());
-
+    if (scanner == nullptr) {
+        // standard flow
+        scanner = std::unique_ptr<InvertedListScanner>(
+                index_ivf->get_InvertedListScanner());
+    }
+    float recall = 0.0;
     for (int i = 0; i < nq; i++) {
         std::vector<idx_t> I(k, -1);
-        float default_dis = metric == METRIC_L2 ? HUGE_VAL : -HUGE_VAL;
+        float default_dis = metric == METRIC_L2
+                ? std::numeric_limits<float>::max()
+                : std::numeric_limits<float>::lowest();
         std::vector<float> D(k, default_dis);
 
-        scanner->set_query(xqt + i * dt);
+        scanner->set_query(xqt + i * index_ivf->d);
 
         for (int j = 0; j < nprobe; j++) {
             int list_no = q_lists[i * nprobe + j];
@@ -202,7 +243,7 @@ void test_lowlevel_access(const char* index_key, MetricType metric) {
                     I.data(),
                     k);
 
-            if (j == 0) {
+            if (distance_to_code_supported && j == 0) {
                 // all results so far come from list_no, so let's check if
                 // the distance function works
                 for (int jj = 0; jj < k; jj++) {
@@ -229,9 +270,57 @@ void test_lowlevel_access(const char* index_key, MetricType metric) {
 
         // check that we have the same results as the reference search
         for (int j = 0; j < k; j++) {
-            EXPECT_EQ(I[j], ref_I[i * k + j]);
+            if (I[j] != ref_I[i * k + j]) {
+                if (accuracy_requirement == 1.0) {
+                    EXPECT_EQ(D[j], ref_D[i * k + j]);
+                }
+                if (D[j] == ref_D[i * k + j]) {
+                    recall += 1.0;
+                }
+            } else {
+                recall += 1.0;
+            }
         }
     }
+    recall = recall / (k * nq);
+    EXPECT_GE(recall, accuracy_requirement);
+}
+
+void test_ivfpqfs_scanner(MetricType metric, bool by_residual = false) {
+    auto index =
+            std::unique_ptr<Index>(index_factory(d, "IVF32,PQ4x4fs", metric));
+    ParameterSpace().set_index_parameter(index.get(), "nprobe", 4);
+    IndexIVFPQFastScan* index_ivf = static_cast<IndexIVFPQFastScan*>(
+            ivflib::extract_index_ivf(index.get()));
+    index_ivf->by_residual = by_residual;
+    // implem_10 also processes one query at a time, so compare with that.
+    index_ivf->implem = 10;
+    auto xt = make_data(nt);
+    index_ivf->train(nt, xt.data());
+    auto xb = make_data(nb);
+    index_ivf->add(nb, xb.data());
+    // Initialize scanner with context in params so heap results can persist
+    // across multiple inverted list scans.
+    auto scanner = std::unique_ptr<InvertedListScanner>(
+            index_ivf->get_InvertedListScanner(true, nullptr, nullptr));
+
+    // ref data
+    auto xq = make_data(nq);
+    auto res = search_index(index_ivf, xq.data());
+
+    // distance_to_code_supported = false because codes are intermixed.
+    test_get_InvertedListScanner(
+            index_ivf,
+            nullptr,
+            {},
+            std::move(scanner),
+            xq,
+            res.first,
+            res.second,
+            metric,
+            false,
+            by_residual ? 0.9 : 1.0 // recall is not 100% for by_residual
+    );
 }
 
 } // anonymous namespace
@@ -276,6 +365,22 @@ TEST(TestLowLevelIVF, IVFRQ) {
     test_lowlevel_access("IVF32,RQ16x8", METRIC_L2);
 }
 
+TEST(TestLowLevelIVF, IVFPQFS_L2) {
+    test_ivfpqfs_scanner(METRIC_L2);
+}
+
+TEST(TestLowLevelIVF, IVFPQFS_IP) {
+    test_ivfpqfs_scanner(METRIC_INNER_PRODUCT);
+}
+
+TEST(TestLowLevelIVF, IVFPQFSr_L2) {
+    test_ivfpqfs_scanner(METRIC_L2, true);
+}
+
+TEST(TestLowLevelIVF, IVFPQFSr_IP) {
+    test_ivfpqfs_scanner(METRIC_INNER_PRODUCT, true);
+}
+
 /*************************************************************
  * Same for binary (a bit simpler)
  *************************************************************/
@@ -459,7 +564,7 @@ void test_threaded_search(const char* index_key, MetricType metric) {
 
     // sample some example queries and get reference search results.
     auto xq = make_data(nq);
-    auto ref_I = search_index(index.get(), xq.data());
+    auto res = search_index(index.get(), xq.data());
 
     // handle preprocessing
     const float* xqt = xq.data();
@@ -570,7 +675,7 @@ void test_threaded_search(const char* index_key, MetricType metric) {
 
         // check that we have the same results as the reference search
         for (int j = 0; j < k; j++) {
-            EXPECT_EQ(I[j], ref_I[i * k + j]);
+            EXPECT_EQ(I[j], res.first[i * k + j]);
         }
     }
 }
